#include <Eigen/Eigen>
#include <mex.h>
#include "r6p.h"



void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]){
	if (nrhs < 3){
		mexErrMsgTxt("r6p: not enough input arguments");
	}
	else{
		const mwSize *dims;
		mxArray * mx;
		double * X, *u, r0;
		dims = mxGetDimensions(prhs[0]);

		int nresults;

		int n_pts = dims[1];
		int pts_dim = dims[0];

		if (n_pts != 6 || pts_dim != 3){
			mexErrMsgTxt("X must be a 3x6 matrix of 3D points");
			return;
		}

		dims = mxGetDimensions(prhs[1]);

		n_pts = dims[1];
		pts_dim = dims[0];

		if (n_pts != 6 || pts_dim != 2){
			mexErrMsgTxt("u must be a 2x6 matrix of 2D projections");
			return;
		}

		X = mxGetPr(prhs[0]);
		u = mxGetPr(prhs[1]);
		r0 = *mxGetPr(prhs[2]);

		double v[60];
		double C[60];
		double t[60];
		double w[60];


		double *v_out, *w_out, *t_out, *C_out;

		nresults = r6p(X, u, r0, C, t, v, w);

		plhs[0] = mxCreateDoubleMatrix(3, nresults, mxREAL);
		plhs[1] = mxCreateDoubleMatrix(3, nresults, mxREAL);
		plhs[2] = mxCreateDoubleMatrix(3, nresults, mxREAL);
		plhs[3] = mxCreateDoubleMatrix(3, nresults, mxREAL);
		plhs[4] = mxCreateDoubleMatrix(9, nresults, mxREAL);

		v_out = mxGetPr(plhs[0]);
		w_out = mxGetPr(plhs[1]);
		t_out = mxGetPr(plhs[2]);
		C_out = mxGetPr(plhs[3]);

		memcpy(v_out, v, nresults * 3 * sizeof(double));
		memcpy(w_out, w, nresults * 3 * sizeof(double));
		memcpy(t_out, t, nresults * 3 * sizeof(double));
		memcpy(C_out, C, nresults * 3 * sizeof(double));

	}



}